SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";
SET time_zone = "+00:00";


/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;

SET SQL_MODE = "NO_AUTO_VALUE_ON_ZERO";
SET time_zone = "+00:00";


delimiter //

-- tr - abbr trigger
-- i - insert
-- u - update
-- d - delete


drop trigger if exists tr_i_submodule//
create trigger tr_i_submodule
    before insert on submodules for each row
begin
    declare vModuleType enum('regular','exam', 'bonus', 'extra') default 'extra';

    -- todo: check repeated bonus
    select modules.Type into vModuleType
        from modules
        where modules.ID = new.ModuleID
        limit 1;

    if ( vModuleType in ('regular', 'exam') ) then begin
        -- only regular & 1st exam submodules can change Cur/Max rates of discipline
        declare vGenModule boolean default (vModuleType != 'exam') or new.OrderNum <=> 1;
        declare vDisciplineID int default 0;

        set vDisciplineID = (
            select modules.DisciplineID from modules
            where modules.ID = new.ModuleID
            limit 1
        );

        update disciplines
        set     disciplines.CurRate = disciplines.CurRate + if(vGenModule and new.isUsed, new.MaxRate, 0),
                disciplines.MaxRate = disciplines.MaxRate + if(vGenModule, new.MaxRate, 0)
        where disciplines.ID = vDisciplineID
        limit 1;
    end; end if;
end//

drop trigger if exists tr_d_submodule//
create trigger tr_d_submodule
    before delete on submodules for each row
begin
    declare vModuleType enum('regular','exam', 'bonus', 'extra') default 'extra';

    -- todo: check repeated bonus
    select modules.Type into vModuleType
        from modules
        where modules.ID = old.ModuleID
        limit 1;


    if ( vModuleType in ('regular', 'exam') ) then begin
        -- only regular & 1st exam submodules can change Cur/Max rates of discipline
        declare vGenModule boolean default (vModuleType != 'exam') or old.OrderNum = 1;
        declare vDisciplineID int default 0;

        set vDisciplineID = (
            select modules.DisciplineID from modules
            where modules.ID = old.ModuleID
            limit 1
        );

        update disciplines
        set     disciplines.CurRate = disciplines.CurRate - if(vGenModule and old.isUsed, old.MaxRate, 0),
                disciplines.MaxRate = disciplines.MaxRate - if(vGenModule, old.MaxRate, 0)
        where disciplines.ID = vDisciplineID
        limit 1;
    end; end if;

end//

drop trigger if exists tr_u_submodule//
create trigger tr_u_submodule
    before update on submodules for each row
begin
    declare vModuleType enum('regular','exam', 'bonus', 'extra') default 'extra';

    -- forbid submodule's module changing
    if old.ModuleID != new.ModuleID then
        signal sqlstate '45000';
    end if;

    -- todo: check repeated bonus
    select modules.Type into vModuleType
        from modules
        where modules.ID = new.ModuleID
        limit 1;


    if ( vModuleType in ('regular', 'exam') ) then begin
        declare vDisciplineID int default 0;
        declare vChangeMaxOld, vChangeMaxNew boolean default false;

        set vDisciplineID = (
            select modules.DisciplineID from modules
            where modules.ID = new.ModuleID
            limit 1
        );

        set vChangeMaxOld = (vModuleType != 'exam') or old.OrderNum = 1;
        set vChangeMaxNew = (vModuleType != 'exam') or new.OrderNum = 1;

        update disciplines
        set     disciplines.CurRate = disciplines.CurRate
                    - if(vChangeMaxOld and old.isUsed, old.MaxRate, 0)
                    + if(vChangeMaxNew and new.isUsed, new.MaxRate, 0),
                disciplines.MaxRate = disciplines.MaxRate
                    - if(vChangeMaxOld, old.MaxRate, 0)
                    + if(vChangeMaxNew, new.MaxRate, 0)
        where disciplines.ID = vDisciplineID
        limit 1;
    end; end if;

end//


delimiter ;


